#!/usr/bin/perl
#
# script by Tim Godfrey
#
# Parse Korg combi definition file "combi.txt" or (V2) "CombiAndSongTimbreSet.txt" into summary parameter information
# used to generate combi portion of KronosParamList.txt for parameter mapping and display
# output format:
# 024000000000000=Cm Insert Effect1 Effect Type
# AAABBBCCCDDDEEE= parameter description string
# AAA=TYP, BBB=SOC, CCC=SUB, DDD=PID, EEE=IDX
# 
# Input file from command line.  perl KorgCombi.pl CombiAndSongTimbreSet.txt outputfile.txt
#
open (INFILE, $ARGV[0]);
 
 

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);

$timestamp = sprintf "%4d%02d%02d-%02d%02d%02d",$year+1900,$mon+1,$mday,$hour,$min,$sec;


if (length($ARGV[1]) > 1)  {    # If a 2nd paramater is given, use it for the output file
   open (OUT, '>'.$ARGV[1]); 
 } else {
   open (OUT, '>'.$timestamp.'.txt');    # Else name the output file with a timestamp
   print "opening $timestamp.txt";
 }

 $bigOFS = 0;    # save largest numbers to print out summary of maximum values
 $bigTYP = 0;
 $bigSOC = 0;
 $bigSUB = 0;
 $bigPID = 0;
 $bigIDX = 0;


 while (<INFILE>) {
  chomp;
  if ($_ =~  m/^\|/)   { # starts with | character

     if ($OFS > $bigOFS) {$bigOFS = $OFS; }       #keep track of largest values
     if ($TYP > $bigTYP) {$bigTYP = $TYP; }  
     if ($SOC > $bigSOC) {$bigSOC = $SOC; }  
     if ($SUB > $bigSUB) {$bigSUB = $SUB; }  
     if ($PID > $bigPID) {$bigPID = $PID; }  
     if ($IDX > $bigIDX) {$bigIDX = $IDX; }  

   #  print OUT "$_\n";   	
     ($bar, $OFS, $bit, $param1, $param2, $data, $valu, $TYP, $SOC, $SUB, $PID, $IDX) = split(/\|/);
     $OFS =~ s/^\s+|\s+$//g;     # trim leading/trailing spaces
     $param1 =~ s/^\s+|\s+$//g;
     $param2 =~ s/^\s+|\s+$//g;
     $valu =~ s/^\s+|\s+$//g;
     $TYP =~ s/^\s+|\s+$//g;
     $SOC =~ s/^\s+|\s+$//g;
     $SUB =~ s/^\s+|\s+$//g;
     $PID =~ s/^\s+|\s+$//g;
     $IDX =~ s/^\s+|\s+$//g;
     
 	
  
     if ($TYP =~  m/\d+/) {          # if TYP is a valid number
     $OFS = sprintf("%03d", $OFS);
     $TYP = sprintf("%03d", $TYP);
     $SOC = sprintf("%03d", $SOC);
     $SUB = sprintf("%03d", $SUB);
     $PID = sprintf("%03d", $PID);
     $IDX = sprintf("%03d", $IDX);
     
     $namestr = $TYP. $SOC . $SUB . $PID . $IDX;
     $namestr =~ s/^\s+|\s+$//g;

#     $valstr = $param1.' '.$param2.' '.$valu.' '.$OFS;
     $valstr = $param1.' '.$param2;
     $valstr =~ s/^\s+|\s+$//g;


     if ($namestr =~  m/\=/)   { # contains = character  - used for name/val separator
       print  "Namestr Warning = character $namestr\n";
       }

     if ($valstr =~  m/\=/)   { # contains = character  - used for name/val separator
       print  "Valstr Warning = character $valstr\n";
       }

     if ((length($namestr) > 0)	and (length($valstr) > 0)) {
       print OUT "$namestr=$valstr\n";  
       }
      else  {
       print "??: $namestr=$valstr\n";  
     }

     }
	   	  
  }

}  # end of while loop

# print out summary of maximum parameter values as sanity check
print  "O $bigOFS  T:$bigTYP SO:$bigSOC  SU:$bigSUB  PI:$bigPID  ID:$bigIDX";


close (INFILE);
close(OUT);

exit;
