#!/usr/bin/perl

# Parse Korg  definition file "effect.txt" into summary parameter information
# script by Tim Godfrey
#
# example command usage:  
# C:\Data\Scripts>perl KorgKronosEffect.pl effect.txt ef_out1.txt
# End: highest bit position: 7, highest offset: 56, highest bitwidth: 15, Total params:, 4019
# C:\Data\Scripts>
#
# sample output line:
# 28,9,7,0,15,"Overdrive/Hyper-Gain Wah","Output Level Mod.Intensity","CE~32","-50~+50"
# EfxProgNum,Offset,BitWidth,BitPosition,PID,"EfxName","EfxParamDescription",'EfxData(Hex)","EfxValue"
#
#
# interpretation of Bit and Parameter (text) field nomencature used in effect.txt:
#	Bit		Param
#	yes		yes			first or only entry for a param		close any previous param
#	no		yes			first or only param of full 8-bit width - close any previous param
#	yes		no			continuation of param with partial bit field
#	no		no			continuation of param with full 8 bit field  (see EFX 04: Ofs:10)
#
#
#  conditional format rule : =AND((F83<>F85-1),(F85>F83))
#
#
#
#
# 
# Input file from command line.
open (INFILE, $ARGV[0]);
 
 

($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);

$timestamp = sprintf "%4d%02d%02d-%02d%02d%02d",$year+1900,$mon+1,$mday,$hour,$min,$sec;


if (length($ARGV[1]) > 1)  {    # If a 2nd paramater is given, use it for the output file
   open (OUT, '>'.$ARGV[1]); 
 } else {
   open (OUT, '>P'.$timestamp.'.txt');    # Else name the output file with a timestamp
   print "opening $timestamp.txt";
 }


if (length($ARGV[1]) > 2)  {    # If a 3rd paramater is given, use it for the 2nd output file
   open (OUT2, '>'.$ARGV[2]); 
 } else {
   open (OUT2, '>E'.$timestamp.'.txt');    # Else name the output file with a timestamp
   print "opening $timestamp.txt";
 }


			$holdline = '';               # place to hold printed line for continued bit field on next byte
			$ofs = '';
			$totalparams = 0;
			$highestbitwidth = 0;
			$lastpid = -1;  # fake value to prevent non-incrementing warning on first parameter
			
 while (<INFILE>) {
  chomp;
  if ($_ =~  m/^\d/)   { # if the line starts with number, it is a new program
		 if (length($ofs) > 0) {
  	 	print OUT2 "$prg, $ofs, \"$prgname\"\n";   	# first print out name previous EFX program and size in bytes (ofs)
  	 	}
  	 $prevprg = $prg;   # save the previous program number
  	 ($prg, $prgname) = split(/\:/);   # separate program number and program name
     $prg =~ s/^\s+|\s+$//g;      # trim lead/trail whitespace
     $prg =~ s/^0+//g ;           # remove leading zero
     $prgname =~ s/^\s+|\s+$//g;  # trim lead/trail whitespace
  	 if ($prg > 1) { #skip the first one since PID isn't known yet
			  	 $totalparams = $totalparams + $lastpid +1 ;  # +1 for zero-based parameter sets
		#	  	 print "Prog $prevprg, Total Params $totalparams, this prg highest param $lastpid\n";
				  	}
     
     
     
#		 print "$prg, $prgname\n";  	

  }
  
  elsif ($_ =~  m/^\|/)   { # starts with | character


#     print OUT "$_\n";   	
     ($bar, $ofs, $bit, $param, $data, $valu, $pid) = split(/\|/);
#  if ($prg == 1) {
#  	 print ",$bar,$ofs,\n";
#  	}
     $ofs =~ s/^\s+//;     # trim leading spaces
     $bit =~ s/^\s+//;
     $param =~ s/^\s+//;
     $data =~ s/^\s+//;
     $valu =~ s/^\s+//;
     $pid =~ s/^\s+//;

     $ofs =~ s/\s+$//;     # trim trailing spaces
     $bit =~ s/\s+$//;
     $param =~ s/\s+$//;
     $data =~ s/\s+$//;
     $valu =~ s/\s+$//;
     $pid =~ s/\s+$//;
     
 	
  
     if (($bit =~  m/\d+/) or (length($bit) == 0)) {          # if bit field contains valid number or is blank (indicating an 8 bit field)


			if (length($ofs) == 0)  {     #if OFS field is empty, use last good value
				$ofs = $currofs;
			}
			else {
				$currofs = $ofs;			#update last when value is present
			}
				
				

			if (length($bit) > 1)  {    #account for bit fields split between lines
				  $hibit = substr($bit,0,1);
				  $lobit = substr($bit,2,1);
				  
				  $bitwidth = $hibit - $lobit + 1;   # calculate width of field from highest and lowest bit specified.
				}
				elsif (length($bit) == 1) {			# a one bit field
				  $hibit = $bit;
				  $lobit = $bit;
				  $bitwidth = 1;
				  } 
				else { 			# a blank field means full 8 bits
				  $hibit = 7;
				  $lobit = 0;
				  $bitwidth = 8;
				}


       $bitpos =  $lobit;
       if ($bitpos > $highestbitpos) {
       	 $highestbitpos = $bitpos;
       }

       if ($ofs > $highestofs) {
       	 $highestofs = $ofs;
       }
       
			if (length($param) > 0)  {    # bit field is complete, or starts here with high bits
				
				  if (length($holdline) > 1) { # this is a new parameter, so if there is a held line, print it
				  	print OUT $holdline;   # print the previosly held line
				  }
			  	$holdline = $prg.",".$ofs.",".$bitwidth.",".$bitpos.",".$pid.",\"".$prgname."\",\"".$param."\",\"".$data."\",\"".$valu."\"\n";    #Build new holdline for this data
				
					$lastofs = $ofs;
				  $lasthibit = $hibit ;      
				  $lastlobit = $lobit;
				  if ($bitwidth > $highestbitwidth) { 
				  	 $highestbitwidth = $bitwidth;
				  	}
				  $lastbitwidth = $bitwidth;
				  $lastbitpos  = $bitpos;
				  $lastparam = $param;
				  $lastdata = $data;
				  $lastvalu = $valu;
				  if (($pid > 0) and ($pid != $lastpid +1 ))  {  
				  	print "Note: for Prg $prg, PID $pid not incremented from previous $lastpid\n";
				  }
				  	
				  $lastpid = $pid;
				  	  
				}
				else {                     # param field is blank, so bit field was continued from previous line ---- update holdline
				  
			  	$lastbitwidth = $lastbitwidth + $bitwidth;  # add in the width of the current bit field on this row to the running total in $lastbitwidth
				  if ($lastbitwidth > $highestbitwidth) {   # keep track of the biggest field of all
				  	 $highestbitwidth = $lastbitwidth;
	 				  }
			  	
			  	# update the holdline with new total bitwidth  --- use previous row values, except updated width
				  $holdline = $prg.",".$lastofs.",".$lastbitwidth.",".$lastbitpos.",".$lastpid.",\"".$prgname."\",\"".$lastparam."\",\"".$lastdata."\",\"".$lastvalu."\"\n";

				}
      

     }  # end of if bit field contains valid number or is blank
	   	  
  }   # end of starts with | character

}  # end of while loop

	  	 $totalparams = $totalparams + $lastpid +1 ;  # include the final program set ----  (+1 for zero-based parameter sets)

			if (length($holdline) > 1) { # print any remaining held line at end
			  	print OUT $holdline;   # print the previosly held line
 			}

print "End: highest bit position: $highestbitpos, highest offset: $highestofs, highest bitwidth: $highestbitwidth, Total params:, $totalparams";
print OUT2 "$prg, $ofs, \"$prgname\"\n";   	# print out size of each EFX program in bytes (ofs)  (for final program)

close (INFILE);
close(OUT);
close(OUT2);

exit;
